@@ -1,9 +1,11 @@
 """Fixtures for use with jupyter server and downstream."""
+
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+from __future__ import annotations
 
-import asyncio
 import importlib
+import inspect
 import io
 import logging
 import os
@@ -11,6 +13,7 @@ import re
 import shutil
 import urllib.parse
 from binascii import hexlify
+from pathlib import Path
 
 import jupyter_core.paths
 import pytest
@@ -31,55 +34,26 @@ try:
     from tornado.websocket import WebSocketHandler
     from traitlets.config import Config
 
-    is_v2 = version_info[0] == 2  # noqa
+    is_v2 = version_info[0] == 2
 
 except ImportError:
-    Authorizer = object  # type:ignore
+    Authorizer = object  # type:ignore[assignment, misc]
     import warnings
 
     warnings.warn(
         "The server plugin has not been installed. "
         "If you're trying to use this plugin and you've installed "
         "`pytest-jupyter`, there is likely one more step "
-        "you need. Try: `pip install 'pytest-jupyter[server]'`"
+        "you need. Try: `pip install 'pytest-jupyter[server]'`",
+        stacklevel=2,
     )
 
 
 # Bring in local plugins.
-from pytest_jupyter.jupyter_core import *  # noqa
-from pytest_jupyter.pytest_tornasync import *  # noqa
+from pytest_jupyter.jupyter_core import *  # noqa: F403
+from pytest_jupyter.pytest_tornasync import *  # noqa: F403
 from pytest_jupyter.utils import mkdir
 
-# Override some of the fixtures from pytest_tornasync
-# The io_loop fixture is overidden in jupyter_core.py so it
-# can be shared by other plugins that need it (e.g. jupyter_client.py).
-
-
-@pytest.fixture
-def http_server(io_loop, http_server_port, jp_web_app):
-    """Start a tornado HTTP server that listens on all available interfaces."""
-
-    async def get_server():
-        """Get a server asynchronously."""
-        server = tornado.httpserver.HTTPServer(jp_web_app)
-        server.add_socket(http_server_port[0])
-        return server
-
-    server = io_loop.run_sync(get_server)
-    yield server
-    server.stop()
-
-    if hasattr(server, "close_all_connections"):
-        try:
-            io_loop.run_sync(server.close_all_connections)
-        except asyncio.TimeoutError:
-            pass
-
-    http_server_port[0].close()
-
-
-# End pytest_tornasync overrides
-
 
 @pytest.fixture
 def jp_server_config():
@@ -113,7 +87,7 @@ def jp_argv():
     return []
 
 
-@pytest.fixture()
+@pytest.fixture
 def jp_http_port(http_server_port):
     """Returns the port value from the http_server_port fixture."""
     yield http_server_port[-1]
@@ -135,7 +109,7 @@ def jp_nbconvert_templates(jp_data_dir):
     possible_paths = jupyter_core.paths.jupyter_path("nbconvert", "templates")
     nbconvert_path = None
     for path in possible_paths:
-        if os.path.exists(path):
+        if Path(path).exists():
             nbconvert_path = path
             break
 
@@ -158,13 +132,13 @@ def jp_logging_stream():
     output = logging_stream.getvalue()
     # If output exists, print it.
     if output:
-        print(output)
+        print(output)  # noqa: T201
     return output
 
 
-@pytest.fixture(scope="function")
+@pytest.fixture
 def jp_configurable_serverapp(
-    jp_nbconvert_templates,  # this fixture must preceed jp_environ
+    jp_nbconvert_templates,  # this fixture must precede jp_environ
     jp_environ,
     jp_server_config,
     jp_argv,
@@ -174,7 +148,6 @@ def jp_configurable_serverapp(
     jp_root_dir,
     jp_logging_stream,
     jp_asyncio_loop,
-    io_loop,
 ):
     """Starts a Jupyter Server instance based on
     the provided configuration values.
@@ -185,8 +158,8 @@ def jp_configurable_serverapp(
     .. code-block:: python
 
       def my_test(jp_configurable_serverapp):
-         app = jp_configurable_serverapp(...)
-         ...
+          app = jp_configurable_serverapp(...)
+          ...
     """
     ServerApp.clear_instance()
 
@@ -204,7 +177,6 @@ def jp_configurable_serverapp(
         environ=jp_environ,
         http_port=jp_http_port,
         tmp_path=tmp_path,
-        io_loop=io_loop,
         root_dir=jp_root_dir,
         **kwargs,
     ):
@@ -226,7 +198,7 @@ def jp_configurable_serverapp(
         app = ServerApp.instance(
             # Set the log level to debug for testing purposes
             log_level="DEBUG",
-            port=jp_http_port,
+            port=http_port,
             port_retries=0,
             open_browser=False,
             base_url=base_url,
@@ -234,7 +206,7 @@ def jp_configurable_serverapp(
             allow_root=True,
             **kwargs,
         )
-        app.init_signal = lambda: None
+        app.init_signal = lambda: None  # type:ignore[method-assign]
         app.log.propagate = True
         app.log.handlers = []
         # Initialize app without httpserver
@@ -245,7 +217,7 @@ def jp_configurable_serverapp(
             async def initialize_app():
                 app.initialize(argv=argv, new_httpserver=False)
 
-            jp_asyncio_loop.run_until_complete(initialize_app())
+            jp_asyncio_loop.run_until_complete(initialize_app())  # type:ignore[no-untyped-call]
         # Reroute all logging StreamHandlers away from stdin/stdout since pytest hijacks
         # these streams and closes them at unfortunate times.
         stream_handlers = [h for h in app.log.handlers if isinstance(h, logging.StreamHandler)]
@@ -259,10 +231,32 @@ def jp_configurable_serverapp(
     return _configurable_serverapp
 
 
-@pytest.fixture(scope="function")
+@pytest.fixture
 def jp_serverapp(jp_server_config, jp_argv, jp_configurable_serverapp):
     """Starts a Jupyter Server instance based on the established configuration values."""
-    return jp_configurable_serverapp(config=jp_server_config, argv=jp_argv)
+    with warnings.catch_warnings():
+        warnings.filterwarnings(
+            "ignore",
+            message=".*WindowsProactorEventLoopPolicy.*",
+            category=DeprecationWarning,
+        )
+        warnings.filterwarnings(
+            "ignore",
+            message=".*WindowsSelectorEventLoopPolicy.*",
+            category=DeprecationWarning,
+        )
+        warnings.filterwarnings(
+            "ignore",
+            message=".*get_event_loop_policy.*",
+            category=DeprecationWarning,
+        )
+        warnings.filterwarnings(
+            "ignore",
+            message=".*set_event_loop_policy.*",
+            category=DeprecationWarning,
+        )
+
+        return jp_configurable_serverapp(config=jp_server_config, argv=jp_argv)
 
 
 @pytest.fixture
@@ -297,6 +291,25 @@ def jp_fetch(jp_serverapp, http_server_client, jp_auth_header, jp_base_url):
         async def my_test(jp_fetch):
             response = await jp_fetch("api", "spec.yaml")
             ...
+
+    With query parameters:
+
+    .. code-block:: python
+
+        async def my_test(jp_fetch):
+            response = await jp_fetch("api", "spec.yaml", params={"parameter": "value"})
+            ...
+
+    A POST request with data:
+
+    .. code-block:: python
+
+        async def my_test(jp_fetch):
+            response = await jp_fetch(
+                "api", "spec.yaml", method="POST", body=json.dumps({"parameter": "value"})
+            )
+            ...
+
     """
 
     def client_fetch(*parts, headers=None, params=None, **kwargs):
@@ -312,8 +325,12 @@ def jp_fetch(jp_serverapp, http_server_client, jp_auth_header, jp_base_url):
         # Add auth keys to header, if not overridden
         for key, value in jp_auth_header.items():
             headers.setdefault(key, value)
+        # Handle timeout
+        request_timeout = kwargs.pop("request_timeout", 20)
         # Make request.
-        return http_server_client.fetch(url, headers=headers, request_timeout=20, **kwargs)
+        return http_server_client.fetch(
+            url, headers=headers, request_timeout=request_timeout, **kwargs
+        )
 
     return client_fetch
 
@@ -330,17 +347,11 @@ def jp_ws_fetch(jp_serverapp, http_server_client, jp_auth_header, jp_http_port,
         async def my_test(jp_fetch, jp_ws_fetch):
             # Start a kernel
             r = await jp_fetch(
-                'api', 'kernels',
-                method='POST',
-                body=json.dumps({
-                    'name': "python3"
-                })
+                "api", "kernels", method="POST", body=json.dumps({"name": "python3"})
             )
-            kid = json.loads(r.body.decode())['id']
+            kid = json.loads(r.body.decode())["id"]
             # Open a websocket connection.
-            ws = await jp_ws_fetch(
-                'api', 'kernels', kid, 'channels'
-            )
+            ws = await jp_ws_fetch("api", "kernels", kid, "channels")
             ...
     """
 
@@ -360,7 +371,11 @@ def jp_ws_fetch(jp_serverapp, http_server_client, jp_auth_header, jp_http_port,
             headers.setdefault(key, value)
         # Make request.
         req = tornado.httpclient.HTTPRequest(url, headers=headers, connect_timeout=120)
-        return tornado.websocket.websocket_connect(req)
+        allowed = list(inspect.signature(tornado.websocket.websocket_connect).parameters)
+        for name in list(kwargs):
+            if name not in allowed:
+                del kwargs[name]
+        return tornado.websocket.websocket_connect(req, **kwargs)
 
     return client_fetch
 
@@ -379,8 +394,8 @@ def jp_create_notebook(jp_root_dir):
         parent = nbpath.parent
         parent.mkdir(parents=True, exist_ok=True)
         # Create a notebook string and write to file.
-        nb = nbformat.v4.new_notebook()
-        nbtext = nbformat.writes(nb, version=4)
+        nb = nbformat.v4.new_notebook()  # type:ignore[no-untyped-call]
+        nbtext = nbformat.writes(nb, version=4)  # type:ignore[no-untyped-call]
         nbpath.write_text(nbtext)
         return nb
 
@@ -395,7 +410,7 @@ def jp_server_cleanup(jp_asyncio_loop):
     try:
         jp_asyncio_loop.run_until_complete(app._cleanup())
     except (RuntimeError, SystemExit) as e:
-        print("ignoring cleanup error", e)
+        print("ignoring cleanup error", e)  # noqa: T201
     if hasattr(app, "kernel_manager"):
         app.kernel_manager.context.destroy()
     ServerApp.clear_instance()
@@ -413,6 +428,7 @@ def send_request(jp_fetch, jp_ws_fetch):
             code = r.code
         except HTTPClientError as err:
             code = err.code
+            print(f"HTTPClientError ({err.code}): {err}")  # noqa: T201
         else:
             if fetch is jp_ws_fetch:
                 r.close()
@@ -451,8 +467,8 @@ class _Authorizer(Authorizer):
     # Set these class attributes from within a test
     # to verify that they match the arguments passed
     # by the REST API.
-    permissions: dict = {}
-    _default_regex_mapping: dict = {}
+    permissions: dict[str, str] = {}
+    _default_regex_mapping: dict[str, str] = {}
 
     HTTP_METHOD_TO_AUTH_ACTION = {
         "GET": "read",
@@ -465,7 +481,9 @@ class _Authorizer(Authorizer):
         "WEBSOCKET": "execute",
     }
 
-    def match_url_to_resource(self, url, regex_mapping=None):
+    def match_url_to_resource(
+        self, url: str, regex_mapping: dict[str, str] | None = None
+    ) -> str | None:
         """Finds the JupyterHandler regex pattern that would
         match the given URL and returns the resource name (str)
         of that handler.
@@ -477,9 +495,13 @@ class _Authorizer(Authorizer):
             pattern = re.compile(regex)
             if pattern.fullmatch(url):
                 return auth_resource
+        return None
 
-    def normalize_url(self, path):
+    def normalize_url(self, path: str) -> str:
         """Drop the base URL and make sure path leads with a /"""
+        if not self.parent:
+            msg = "Cannot normalize the url without a parent object"
+            raise ValueError(msg)
         base_url = self.parent.base_url
         # Remove base_url
         if path.startswith(base_url):
@@ -489,7 +511,7 @@ class _Authorizer(Authorizer):
             path = "/" + path
         return path
 
-    def is_authorized(self, handler, user, action, resource):
+    def is_authorized(self, handler, user, action, resource):  # noqa: ARG002
         """Test if a request is authorized."""
         # Parse Request
         method = "WEBSOCKET" if isinstance(handler, WebSocketHandler) else handler.request.method
